import { jsx } from 'react/jsx-runtime';
import React, { useState, useCallback, useEffect, useRef } from 'react';
import styled from '@emotion/styled';

const useAudioContext = (audioUrl, useMicrophone, fftSize = 2048, smoothingTimeConstant = 0.8, minDecibels = -90, maxDecibels = -10) => {
    const [audioState, setAudioState] = useState({
        audioContext: null,
        analyser: null,
        dataArray: null,
        source: null,
    });
    const createAnalyser = (audioContext) => {
        const analyser = audioContext.createAnalyser();
        analyser.fftSize = fftSize;
        analyser.smoothingTimeConstant = smoothingTimeConstant;
        analyser.minDecibels = minDecibels;
        analyser.maxDecibels = maxDecibels;
        return analyser;
    };
    const handleMicrophoneInput = async (audioContext, analyser) => {
        const stream = await navigator.mediaDevices.getUserMedia({ audio: true });
        const source = audioContext.createMediaStreamSource(stream);
        source.connect(analyser);
        setAudioState({ audioContext, analyser, dataArray: new Uint8Array(analyser.frequencyBinCount), source });
    };
    const handleAudioUrlInput = async (audioContext, analyser) => {
        if (!audioUrl) {
            return;
        }
        try {
            const response = await fetch(audioUrl);
            if (!response.ok) {
                throw new Error(`HTTP error! status: ${response.status}`);
            }
            const contentType = response.headers.get('content-type');
            if (!contentType || !contentType.includes('audio')) {
                throw new Error(`Unsupported content type: ${contentType}`);
            }
            const arrayBuffer = await response.arrayBuffer();
            const audioBuffer = await audioContext.decodeAudioData(arrayBuffer);
            const source = audioContext.createBufferSource();
            source.buffer = audioBuffer;
            source.connect(analyser);
            analyser.connect(audioContext.destination);
            source.start(0);
            setAudioState({ audioContext, analyser, dataArray: new Uint8Array(analyser.frequencyBinCount), source });
        }
        catch (error) {
            console.error('Error handling audio URL input:', error);
        }
    };
    const inputHandlers = {
        microphone: handleMicrophoneInput,
        audioUrl: handleAudioUrlInput,
    };
    const initializeAudioContext = useCallback(async () => {
        try {
            const audioContext = new (window.AudioContext || window.webkitAudioContext)();
            const analyser = createAnalyser(audioContext);
            const inputType = useMicrophone ? 'microphone' : 'audioUrl';
            const handler = inputHandlers[inputType];
            if (handler) {
                await handler(audioContext, analyser);
            }
        }
        catch (error) {
            console.error('Error initializing audio context:', error);
        }
    }, [audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels]);
    const cleanup = useCallback(() => {
        const stopSource = (source) => {
            if (source instanceof AudioBufferSourceNode) {
                source.stop();
            }
        };
        const disconnectNode = (node) => {
            node?.disconnect();
        };
        const closeContext = (context) => {
            context?.close();
        };
        stopSource(audioState.source);
        disconnectNode(audioState.source);
        disconnectNode(audioState.analyser);
        closeContext(audioState.audioContext);
    }, [audioState]);
    useEffect(() => {
        initializeAudioContext();
        return () => cleanup();
    }, [initializeAudioContext]);
    return audioState;
};

const useCanvasVisualizer = (audioUrl, useMicrophone, width, height, fftSize, smoothingTimeConstant, minDecibels, maxDecibels, drawCallback) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    useEffect(() => {
        const draw = () => {
            if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
                return;
            const ctx = canvasRef.current.getContext("2d");
            if (!ctx)
                return;
            const bufferLength = audioState.analyser.frequencyBinCount;
            const dataArray = new Uint8Array(bufferLength);
            audioState.analyser.getByteFrequencyData(dataArray);
            ctx.clearRect(0, 0, width, height);
            drawCallback(ctx, dataArray, bufferLength);
            animationFrameId.current = requestAnimationFrame(draw);
        };
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState, width, height, drawCallback]);
    return canvasRef;
};

const createGradient = (ctx, type, colors, width, height, radius) => {
    if (!colors || colors.length < 2)
        return colors[0];
    let gradient;
    if (type === "linear") {
        gradient = ctx.createLinearGradient(0, height, 0, 0);
    }
    else {
        gradient = ctx.createRadialGradient(width / 2, height / 2, 0, width / 2, height / 2, radius || 0);
    }
    colors.forEach((color, index) => {
        gradient.addColorStop(index / (colors.length - 1), color);
    });
    return gradient;
};

const StyledCanvas = styled.canvas `
  width: 100%;
  height: 100%;
`;
const Container = styled.div `
  position: relative;
  width: 100%;
  height: 100%;
  background: linear-gradient(180deg, ${props => props.backgroundColor} 0%, #16213e 100%);
  display: flex;
  justify-content: center;
  align-items: center;
`;
const Canvas = React.forwardRef((props, ref) => (jsx(StyledCanvas, { ...props, ref: ref })));

const WaveformVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 200, backgroundColor = '#1a1a2e', gradientColors = ['#00bcd4', '#4CAF50', '#8BC34A'], barWidth = 3, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, }) => {
    const drawWaveform = (ctx, dataArray, bufferLength) => {
        ctx.lineWidth = barWidth;
        ctx.strokeStyle = createGradient(ctx, 'linear', gradientColors, width, height);
        ctx.beginPath();
        const sliceWidth = (width * 1.0) / bufferLength;
        let x = 0;
        for (let i = 0; i < bufferLength; i++) {
            const v = dataArray[i] / 128.0;
            const y = (v * height) / 2;
            if (i === 0) {
                ctx.moveTo(x, y);
            }
            else {
                ctx.lineTo(x, y);
            }
            x += sliceWidth;
        }
        ctx.lineTo(width, height / 2);
        ctx.stroke();
    };
    const canvasRef = useCanvasVisualizer(audioUrl || '', useMicrophone || false, width, height, fftSize, smoothingTimeConstant, minDecibels, maxDecibels, drawWaveform);
    return (jsx(Container, { backgroundColor: backgroundColor, children: jsx(Canvas, { ref: canvasRef }) }));
};

const FrequencyVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 200, backgroundColor = '#1a1a2e', gradientColors = ['#00bcd4', '#4CAF50', '#8BC34A'], barWidth = 6, barSpacing = 2, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, }) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    const draw = () => {
        if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
            return;
        const canvas = canvasRef.current;
        const ctx = canvas.getContext('2d');
        if (!ctx)
            return;
        const bufferLength = audioState.analyser.frequencyBinCount;
        const dataArray = new Uint8Array(bufferLength);
        audioState.analyser.getByteFrequencyData(dataArray);
        ctx.clearRect(0, 0, width, height);
        const barCount = Math.min(bufferLength, Math.floor(width / (barWidth + barSpacing)));
        const totalWidth = barCount * (barWidth + barSpacing);
        const startX = (width - totalWidth) / 2;
        ctx.fillStyle = createGradient(ctx, 'linear', gradientColors, width, height);
        for (let i = 0; i < barCount; i++) {
            const barHeight = (dataArray[i] / 255.0) * height;
            const x = startX + i * (barWidth + barSpacing);
            const y = height - barHeight;
            ctx.fillRect(x, y, barWidth, barHeight);
        }
        animationFrameId.current = requestAnimationFrame(draw);
    };
    useEffect(() => {
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState, width, height]);
    return (jsx(Container, { backgroundColor: backgroundColor, children: jsx(Canvas, { ref: canvasRef }) }));
};

const CircularVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 800, backgroundColor = '#1a1a2e', gradientColors = ['#00bcd4', '#4CAF50', '#8BC34A'], barWidth = 3, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, animationSpeed = 1, }) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    const rotation = useRef(0);
    const draw = () => {
        if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
            return;
        const canvas = canvasRef.current;
        const ctx = canvas.getContext('2d');
        if (!ctx)
            return;
        const bufferLength = audioState.analyser.frequencyBinCount;
        const dataArray = new Uint8Array(bufferLength);
        audioState.analyser.getByteFrequencyData(dataArray);
        ctx.clearRect(0, 0, width, height);
        const centerX = width / 2;
        const centerY = height / 2;
        const radius = Math.min(width, height) / 3;
        const bars = Math.min(bufferLength, 360);
        const angleStep = (2 * Math.PI) / bars;
        rotation.current += 0.002 * animationSpeed;
        if (rotation.current >= 2 * Math.PI) {
            rotation.current = 0;
        }
        ctx.save();
        ctx.translate(centerX, centerY);
        ctx.rotate(rotation.current);
        for (let i = 0; i < bars; i++) {
            const angle = i * angleStep;
            const magnitude = dataArray[i] / 255.0;
            const barHeight = magnitude * radius;
            const x1 = Math.cos(angle) * radius;
            const y1 = Math.sin(angle) * radius;
            const x2 = Math.cos(angle) * (radius + barHeight);
            const y2 = Math.sin(angle) * (radius + barHeight);
            ctx.strokeStyle = createGradient(ctx, 'radial', gradientColors, width, height, radius + barHeight);
            ctx.lineWidth = barWidth;
            ctx.beginPath();
            ctx.moveTo(x1, y1);
            ctx.lineTo(x2, y2);
            ctx.stroke();
        }
        ctx.restore();
        animationFrameId.current = requestAnimationFrame(draw);
    };
    useEffect(() => {
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState, width, height]);
    return (jsx(Container, { backgroundColor: backgroundColor, children: jsx(Canvas, { ref: canvasRef }) }));
};

var VisualizerType;
(function (VisualizerType) {
    VisualizerType["Waveform"] = "waveform";
    VisualizerType["Frequency"] = "frequency";
    VisualizerType["Circular"] = "circular";
})(VisualizerType || (VisualizerType = {}));

export { CircularVisualizer, FrequencyVisualizer, VisualizerType, WaveformVisualizer, useAudioContext };
//# sourceMappingURL=index.esm.js.map
